const fs = require('fs-extra')
const path = require('path')
const ora = require('ora')
const kebabCase = require('lodash.kebabcase')
const { print } = require('@toptal/davinci-cli-shared')

const workspace = require('../utils/workspace')
const skeleton = require('../utils/skeleton')
const {
  LIBS_PATH,
  DOMAINS_PATH
} = require('../constants')

const getLibPath = (libName, domain) =>
  domain
    ? path.join(DOMAINS_PATH, domain, LIBS_PATH, libName)
    : path.join(LIBS_PATH, libName)

const checkPrerequisites = async (
  monorepoWorkspaceDir,
  libName,
  domain
) => {
  const spinner = ora('Checking prerequisites...').start()

  // check if run in monorepo
  const isWorkspace = Boolean(monorepoWorkspaceDir)
  if (!isWorkspace) {
    spinner.fail()
    throw new Error("It looks like you run 'new-lib' Davinci command not in a monorepo.")
  }

  // check if app already exists
  const libPath = path.join(monorepoWorkspaceDir, getLibPath(libName, domain))
  const doesLibExist = await fs.exists(libPath)

  if (doesLibExist) {
    spinner.fail()
    throw new Error(`Library already exists in the folder ${libPath}!`)
  }

  spinner.succeed('Prerequisites checked!')
}

const copyLibrarySkeleton = async (destDir, currentDavinciRunningDir) => {
  const skeletonLibPath = require.resolve(
    '@toptal/davinci-skeleton-lib',
    {
      paths: [currentDavinciRunningDir]
    }
  )
  const appDir = path.join(skeletonLibPath, '../..')

  const spinner = ora('Copying skeleton...').start()

  try {
    await skeleton.copyAllFiles(appDir, destDir)
    await skeleton.copyTemplatePackageJsonFile(destDir)

    spinner.succeed('Skeleton copied!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const modifyPackageJson = async (destDir, libName, domain, workspacePackageName) => {
  const spinner = ora('Modifiying package.json...').start()

  try {
    const config = skeleton.readPackageJson(destDir)

    config.name = domain ? `@toptal/${domain}-${libName}` : `@toptal/${libName}`
    config.homepage = `https://github.com/toptal/${workspacePackageName}/tree/master/${getLibPath(libName, domain)}`

    delete config.gitHead

    delete config.publishConfig
    config.license = 'SEE LICENSE IN LICENSE.MD'

    config.version = '0.0.1'

    skeleton.writePackageJson(destDir, config)

    spinner.succeed('package.json modified!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const installDependencies = async monorepoWorkspaceDir => {
  const spinner = ora('Installing dependencies... ').start()

  try {
    await workspace.installDependencies(monorepoWorkspaceDir)

    spinner.succeed('Dependencies installed!')
  } catch (err) {
    spinner.fail()
    throw new Error(err)
  }
}

const createCommand = async (libName, domain) => {
  print.green(
    'Creating',
    libName,
    domain ? `in domain ${domain}` : '',
    '...'
  )

  const monorepoWorkspaceDir = workspace.getRootDir()
  const workspacePackageJson = workspace.readPackageJson(monorepoWorkspaceDir)
  const currentDavinciRunningDir = __dirname

  try {
    // check if all apps are installed, directory doesn't exists, etc.
    await checkPrerequisites(
      monorepoWorkspaceDir,
      libName,
      domain
    )

    const destDir = path.join(monorepoWorkspaceDir, getLibPath(libName, domain))
    const workspacePackageName = workspace.getPackageName(workspacePackageJson)

    // copy skeleton to destination directory
    await copyLibrarySkeleton(destDir, currentDavinciRunningDir)

    // change package.json
    await modifyPackageJson(destDir, libName, domain, workspacePackageName)

    // run yarn
    await installDependencies(monorepoWorkspaceDir)

    print.green('Library has been created.')
  } catch (err) {
    print.red(err)
    process.exit(1)
  }
}


const newLibCommandCreator = {
  command: 'new-lib <libName>',
  description: 'Create new library in monorepo',
  options: [{
    label: "Domain of the lib. Creates the lib in a monorepo domain: './domains/{domain}/libs/{libName}'",
    name: '--domain <domain>'
  }],
  action: async ({ libName, options: { domain } }) => {
    const formattedLibName = kebabCase(libName)

    return createCommand(formattedLibName, domain)
  }
}

module.exports = newLibCommandCreator
